@@ -1,5 +1,5 @@
 /* app.c: Application context routines for gzochid
- * Copyright (C) 2012 Julian Graham
+ * Copyright (C) 2013 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 
 #include <glib.h>
 #include <stdlib.h>
+#include <sys/time.h>
 
 #include "app.h"
 #include "auth.h"
@@ -34,9 +35,13 @@
 #include "tx.h"
 #include "util.h"
 
+#define GZOCHID_APPLICATION_MAX_ATTEMPTS_DEFAULT 3
+
 static GStaticPrivate thread_application_context_key = G_STATIC_PRIVATE_INIT;
 static GStaticPrivate thread_identity_key = G_STATIC_PRIVATE_INIT;
 
+G_LOCK_DEFINE_STATIC (load_path);
+
 static void initialize_async (gpointer data, gpointer user_data)
 {
   gzochid_context *context = (gzochid_context *) data;
@@ -76,6 +81,31 @@ static void initialize_data (int from_state, int to_state, gpointer user_data)
   free (names_db);
 }
 
+static void initialize_load_paths_guile_worker
+(gpointer data, gpointer user_data)
+{
+  GList *load_path_ptr = (GList *) data;
+
+  while (load_path_ptr != NULL)
+    {
+      gzochid_scheme_append_load_path ((char *) load_path_ptr->data);
+      load_path_ptr = load_path_ptr->next;
+    }
+}
+
+static void initialize_load_paths
+(int from_state, int to_state, gpointer user_data)
+{
+  gzochid_context *context = (gzochid_context *) user_data;
+  gzochid_application_context *app_context = 
+    (gzochid_application_context *) context;
+  
+  G_LOCK (load_path);
+  gzochid_guile_run 
+    (initialize_load_paths_guile_worker, app_context->descriptor->load_paths);
+  G_UNLOCK (load_path);
+}
+
 static void initialize_complete 
 (int from_state, int to_state, gpointer user_data)
 {
@@ -91,6 +121,72 @@ static void initialize_complete
     (game_context->pool, initialize_async, user_data, NULL);
 }
 
+gzochid_application_task *gzochid_deserialize_application_task 
+(gzochid_application_context *context, 
+ gzochid_application_task_serialization *serialization, GString *in)
+{
+  gzochid_auth_identity *identity = 
+    gzochid_auth_identity_deserializer (context, in);
+  gzochid_application_worker worker = 
+    serialization->worker_serialization->deserializer (context, in);
+  gpointer data = serialization->data_serialization->deserializer (context, in);
+  return gzochid_application_task_new (context, identity, worker, data);
+}
+
+void gzochid_serialize_application_task
+(gzochid_application_context *context,
+ gzochid_application_task_serialization *serialization, 
+ gzochid_application_task *task, GString *out)
+{
+  gzochid_auth_identity_serializer (context, task->identity, out);
+  serialization->worker_serialization->serializer (context, task->worker, out);
+  serialization->data_serialization->serializer (context, task->data, out);
+}
+
+static gzochid_transactional_application_task_execution *execution_new
+(gzochid_application_task *task, struct timeval *timeout)
+{
+  gzochid_transactional_application_task_execution *execution = 
+    malloc (sizeof (gzochid_transactional_application_task_execution));
+
+  execution->task = task;
+
+  if (timeout != NULL)
+    {
+      execution->timeout = malloc (sizeof (struct timeval));
+      execution->timeout->tv_sec = timeout->tv_sec;
+      execution->timeout->tv_usec = timeout->tv_usec;
+    }
+  else execution->timeout = NULL;
+
+  execution->attempts = 0;
+  execution->result = GZOCHID_TRANSACTION_PENDING;
+
+  return execution;
+}
+
+gzochid_transactional_application_task_execution *
+gzochid_transactional_application_task_execution_new 
+(gzochid_application_task *task)
+{
+  return execution_new (task, NULL);
+}
+
+gzochid_transactional_application_task_execution *
+gzochid_transactional_application_task_timed_execution_new 
+(gzochid_application_task *task, struct timeval timeout)
+{
+  return execution_new (task, &timeout);
+}
+
+void gzochid_transactional_application_task_execution_free
+(gzochid_transactional_application_task_execution *execution)
+{
+  if (execution->timeout != NULL)
+    free (execution->timeout);
+  free (execution);
+}
+
 static void transactional_task_worker (gpointer data)
 {
   void **args = (void **) data;
@@ -98,8 +194,7 @@ static void transactional_task_worker (gpointer data)
   gzochid_application_context *context = 
     (gzochid_application_context *) args[0];
   gzochid_auth_identity *identity = (gzochid_auth_identity *) args[1];
-  gzochid_transactional_application_task *task = 
-    (gzochid_transactional_application_task *) args[2];
+  gzochid_application_task *task = (gzochid_application_task *) args[2];
   
   task->worker (context, identity, task->data);
 }
@@ -109,12 +204,57 @@ void gzochid_application_transactional_task_worker
  gpointer data)
 {
   void *args[3];
-  
+  gzochid_transactional_application_task_execution *execution = 
+    (gzochid_transactional_application_task_execution *) data;
+
   args[0] = context;
   args[1] = identity;
-  args[2] = data;
+  args[2] = execution->task;
 
-  gzochid_transaction_execute (transactional_task_worker, args);  
+  if (execution->timeout != NULL)
+    execution->result = gzochid_transaction_execute_timed 
+      (transactional_task_worker, args, *execution->timeout);
+  else execution->result = 
+	 gzochid_transaction_execute (transactional_task_worker, args);
+
+  execution->attempts++;
+}
+
+void gzochid_application_resubmitting_transactional_task_worker 
+(gzochid_application_context *app_context, gzochid_auth_identity *identity, 
+ gpointer data)
+{
+  gzochid_transactional_application_task_execution *execution = 
+    (gzochid_transactional_application_task_execution *) data;
+
+  gzochid_application_transactional_task_worker 
+    (app_context, identity, execution);
+  
+  if (execution->result != GZOCHID_TRANSACTION_SUCCESS
+      && gzochid_application_should_retry (execution))
+    {
+      struct timeval now;
+      gzochid_context *context = (gzochid_context *) app_context;
+      gzochid_game_context *game_context = 
+	(gzochid_game_context *) context->parent;
+
+      gzochid_application_task *application_task = NULL;
+      gzochid_task *task = NULL;
+
+      execution->result = GZOCHID_TRANSACTION_PENDING;
+
+      application_task = gzochid_application_task_new
+	(app_context, identity,
+	 gzochid_application_resubmitting_transactional_task_worker, 
+	 execution);
+
+      gettimeofday (&now, NULL);
+
+      task = gzochid_task_new
+	(gzochid_application_task_thread_worker, application_task, now);
+
+      gzochid_schedule_submit_task (game_context->task_queue, task);
+    }
 }
 
 void gzochid_application_task_worker (gpointer data)
@@ -128,6 +268,13 @@ void gzochid_application_task_thread_worker (gpointer data, gpointer user_data)
   gzochid_application_task_worker (data);
 }
 
+gboolean gzochid_application_should_retry 
+(gzochid_transactional_application_task_execution *execution)
+{
+  return execution->result == GZOCHID_TRANSACTION_SHOULD_RETRY
+    && execution->attempts < GZOCHID_APPLICATION_MAX_ATTEMPTS_DEFAULT;
+}
+
 static void run_async_transactional (gpointer data)
 {
   gzochid_application_context *context = (gzochid_application_context *) data;
@@ -222,8 +369,21 @@ static gpointer deserialize_callback
   return callback;
 }
 
+static void finalize_callback 
+(gzochid_application_context *context, gpointer data)
+{
+  gzochid_application_callback *callback = 
+    (gzochid_application_callback *) data;
+
+  free (callback->procedure);
+  g_list_free_full (callback->module, free);
+  mpz_clear (callback->scm_oid);
+
+  free (callback);
+}
+
 gzochid_io_serialization gzochid_application_callback_serialization = 
-  { serialize_callback, deserialize_callback };
+  { serialize_callback, deserialize_callback, finalize_callback };
 
 gzochid_application_callback *gzochid_application_callback_new
 (char *procedure, GList *module, mpz_t scm_oid)
@@ -287,7 +447,8 @@ void gzochid_application_context_init
     (fsm, GZOCHID_APPLICATION_STATE_INITIALIZING, 
      GZOCHID_APPLICATION_STATE_RUNNING);
   gzochid_fsm_add_transition
-    (fsm, GZOCHID_APPLICATION_STATE_RUNNING, GZOCHID_APPLICATION_STATE_STOPPED);
+    (fsm, GZOCHID_APPLICATION_STATE_RUNNING, 
+     GZOCHID_APPLICATION_STATE_STOPPED);
   gzochid_fsm_add_transition
     (fsm, GZOCHID_APPLICATION_STATE_RUNNING, GZOCHID_APPLICATION_STATE_PAUSED);
   gzochid_fsm_add_transition
@@ -298,7 +459,11 @@ void gzochid_application_context_init
   gzochid_fsm_on_enter 
     (fsm, GZOCHID_APPLICATION_STATE_INITIALIZING, initialize_data, context);
   gzochid_fsm_on_enter 
-    (fsm, GZOCHID_APPLICATION_STATE_INITIALIZING, initialize_complete, context);
+    (fsm, GZOCHID_APPLICATION_STATE_INITIALIZING, initialize_load_paths, 
+     context);
+  gzochid_fsm_on_enter 
+    (fsm, GZOCHID_APPLICATION_STATE_INITIALIZING, initialize_complete, 
+     context);
   gzochid_fsm_on_enter (fsm, GZOCHID_APPLICATION_STATE_RUNNING, run, context);
   gzochid_fsm_on_enter (fsm, GZOCHID_APPLICATION_STATE_STOPPED, stop, context);
 
@@ -316,8 +481,12 @@ void gzochid_application_client_logged_in
   gzochid_client_session *session = 
     gzochid_client_session_new (client->identity);
 
-  gzochid_transactional_application_task transactional_task;
+  gzochid_application_task transactional_task;
   gzochid_application_task application_task;
+  gzochid_transactional_application_task_execution *execution =
+    gzochid_transactional_application_task_timed_execution_new 
+    (&transactional_task, game_context->tx_timeout);
+
   gzochid_task task;
 
   char *session_oid_str = NULL;
@@ -327,22 +496,48 @@ void gzochid_application_client_logged_in
   gzochid_client_session_persist (context, session, session_oid);
   session_oid_str = mpz_get_str (NULL, 16, session_oid);
 
+  g_mutex_lock (context->client_mapping_lock);
   g_hash_table_insert (context->oids_to_clients, session_oid_str, client);
   g_hash_table_insert (context->clients_to_oids, client, session_oid_str);
+  g_mutex_unlock (context->client_mapping_lock);
 
   transactional_task.worker = gzochid_scheme_application_logged_in_worker;
+  transactional_task.context = context;
+  transactional_task.identity = client->identity;
   transactional_task.data = session_oid_str;
 
   application_task.worker = gzochid_application_transactional_task_worker;
   application_task.context = context;
   application_task.identity = client->identity;
-  application_task.data = &transactional_task;
+  application_task.data = execution;
 
   task.worker = gzochid_application_task_thread_worker;
   task.data = &application_task;
   gettimeofday (&task.target_execution_time, NULL);
 
-  gzochid_schedule_run_task (game_context->task_queue, &task);
+  while (TRUE)
+    {
+      gzochid_schedule_run_task (game_context->task_queue, &task);
+      if (!gzochid_application_should_retry (execution))
+	{
+	  if (execution->result != GZOCHID_TRANSACTION_SUCCESS)	
+	    {
+	      gzochid_info
+		("Disconnecting session '%s'; failed login transaction.", 
+		 session_oid_str);
+
+	      g_mutex_lock (context->client_mapping_lock);
+	      g_hash_table_remove (context->oids_to_clients, session_oid_str);
+	      g_hash_table_remove (context->clients_to_oids, client);
+	      g_mutex_unlock (context->client_mapping_lock);
+
+	      gzochid_protocol_client_disconnect (client);
+	    }
+	    break;
+	}
+    }
+
+  gzochid_transactional_application_task_execution_free (execution);
 }
 
 void gzochid_application_client_disconnected
@@ -362,35 +557,40 @@ void gzochid_application_client_disconnected
     }
   else 
     {
-      gzochid_transactional_application_task *transactional_task =
-	gzochid_transactional_application_task_new
-	(gzochid_scheme_application_disconnected_worker, session_oid_str);
-
-      gzochid_application_task *application_task =
+      gzochid_application_task *transactional_task =
 	gzochid_application_task_new
 	(context, client->identity, 
-	 gzochid_application_transactional_task_worker, transactional_task);
+	 gzochid_scheme_application_disconnected_worker, session_oid_str);
+      gzochid_transactional_application_task_execution *execution = 
+	gzochid_transactional_application_task_timed_execution_new
+	(transactional_task, game_context->tx_timeout);
+      gzochid_application_task *application_task = gzochid_application_task_new
+	(context, client->identity,
+	 gzochid_application_resubmitting_transactional_task_worker, 
+	 execution);
 
       gzochid_task *task = NULL;
       struct timeval now;
+
       gettimeofday (&now, NULL);
 
       g_hash_table_remove (context->clients_to_oids, client);
       g_hash_table_remove (context->oids_to_clients, session_oid_str);
       
-      task = gzochid_task_new 
-	(gzochid_application_task_thread_worker, application_task, now);
+      task = gzochid_task_new
+	(gzochid_application_task_thread_worker, application_task,
+	 now);
       
       gzochid_schedule_submit_task (game_context->task_queue, task);
       g_mutex_unlock (context->client_mapping_lock);
     }
 }
 
-gzochid_application_worker_serialization received_message_worker_serialization =
-  { NULL, NULL };
+gzochid_application_worker_serialization 
+received_message_worker_serialization = { NULL, NULL };
 
 gzochid_io_serialization received_message_data_serialization = 
-  { NULL, NULL };
+  { NULL, NULL, NULL };
 
 gzochid_application_task_serialization 
 gzochid_client_received_message_task_serialization = 
@@ -418,8 +618,11 @@ void gzochid_application_session_received_message
     return;
   else 
     {
-      gzochid_transactional_application_task transactional_task;
+      gzochid_application_task transactional_task;
       gzochid_application_task application_task;
+      gzochid_transactional_application_task_execution *execution =
+	gzochid_transactional_application_task_timed_execution_new
+	(&transactional_task, game_context->tx_timeout);
       gzochid_task task;
 
       data[0] = session_oid_str;
@@ -428,18 +631,27 @@ void gzochid_application_session_received_message
       
       transactional_task.worker = 
 	gzochid_scheme_application_received_message_worker;
+      transactional_task.context = context;
+      transactional_task.identity = client->identity;
       transactional_task.data = data;
       
       application_task.worker = gzochid_application_transactional_task_worker;
       application_task.context = context;
       application_task.identity = client->identity;
-      application_task.data = &transactional_task;
+      application_task.data = execution;
       
       task.worker = gzochid_application_task_thread_worker;
       task.data = &application_task;
       gettimeofday (&task.target_execution_time, NULL);
 
-      gzochid_schedule_run_task (game_context->task_queue, &task);
+      while (TRUE)
+	{
+	  gzochid_schedule_run_task (game_context->task_queue, &task);
+	  if (!gzochid_application_should_retry (execution))
+	    break;
+	}
+
+      gzochid_transactional_application_task_execution_free (execution);
     }
 }
 
@@ -494,3 +706,9 @@ gzochid_auth_identity *gzochid_get_current_identity (void)
 {
   return (gzochid_auth_identity *) g_static_private_get (&thread_identity_key);
 }
+
+void gzochid_register_client_received_message_task_serialization (void)
+{
+  gzochid_task_register_serialization 
+    ("received-message", &gzochid_client_received_message_task_serialization);
+}
