@@ -1,5 +1,5 @@
 /* app.h: Prototypes and declarations for app.c
- * Copyright (C) 2011 Julian Graham
+ * Copyright (C) 2013 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -20,12 +20,13 @@
 
 #include <glib.h>
 #include <gmp.h>
-#include <libguile.h>
+#include <sys/time.h>
 
 #include "auth.h"
 #include "context.h"
 #include "io.h"
 #include "storage.h"
+#include "tx.h"
 
 enum gzochid_application_state 
   {
@@ -79,11 +80,14 @@ typedef struct _gzochid_application_context
 typedef void (*gzochid_application_worker) 
 (gzochid_application_context *, gzochid_auth_identity *, gpointer);
 
-void gzochid_application_transactional_task_worker 
-(gzochid_application_context *, gzochid_auth_identity *, gpointer);
+typedef struct _gzochid_application_task
+{
+  gzochid_application_worker worker;
+  gzochid_application_context *context;
+  gzochid_auth_identity *identity;
+  gpointer data;
 
-void gzochid_application_task_worker (gpointer);
-void gzochid_application_task_thread_worker (gpointer, gpointer);
+} gzochid_application_task;
 
 typedef struct _gzochid_application_worker_serialization
 {
@@ -100,14 +104,44 @@ typedef struct _gzochid_application_task_serialization
   gzochid_io_serialization *data_serialization;
 } gzochid_application_task_serialization;
 
-typedef struct _gzochid_application_task
+typedef struct _gzochid_transactional_application_task_execution
 {
-  gzochid_application_worker worker;
-  gzochid_application_context *context;
-  gzochid_auth_identity *identity;
-  gpointer data;
+  gzochid_application_task *task;
 
-} gzochid_application_task;
+  struct timeval *timeout;
+  unsigned int attempts;
+  gzochid_transaction_result result;
+} gzochid_transactional_application_task_execution;
+
+gzochid_application_task *gzochid_deserialize_application_task 
+(gzochid_application_context *, gzochid_application_task_serialization *, 
+ GString *);
+
+void gzochid_serialize_application_task 
+(gzochid_application_context *, gzochid_application_task_serialization *, 
+ gzochid_application_task *, GString *);
+
+gzochid_transactional_application_task_execution *
+gzochid_transactional_application_task_execution_new 
+(gzochid_application_task *);
+
+gzochid_transactional_application_task_execution *
+gzochid_transactional_application_task_timed_execution_new 
+(gzochid_application_task *, struct timeval);
+
+void gzochid_transactional_application_task_execution_free
+(gzochid_transactional_application_task_execution *);
+
+void gzochid_application_transactional_task_worker 
+(gzochid_application_context *, gzochid_auth_identity *, gpointer);
+void gzochid_application_resubmitting_transactional_task_worker 
+(gzochid_application_context *, gzochid_auth_identity *, gpointer);
+
+void gzochid_application_task_worker (gpointer);
+void gzochid_application_task_thread_worker (gpointer, gpointer);
+
+gboolean gzochid_application_should_retry 
+(gzochid_transactional_application_task_execution *);
 
 typedef struct _gzochid_durable_application_task
 {
@@ -122,12 +156,6 @@ typedef struct _gzochid_durable_application_task
   mpz_t oid;
 } gzochid_durable_application_task;
 
-typedef struct _gzochid_transactional_application_task
-{
-  gzochid_application_worker worker;
-  gpointer data;
-} gzochid_transactional_application_task;
-
 gzochid_io_serialization gzochid_application_callback_serialization;
 
 gzochid_application_callback *gzochid_application_callback_new 
@@ -142,6 +170,8 @@ void gzochid_application_context_init
 (gzochid_application_context *, gzochid_context *, 
  gzochid_application_descriptor *);
 
+void gzochid_register_client_received_message_task_serialization (void);
+
 struct _gzochid_protocol_client;
 
 void gzochid_application_client_logged_in
