@@ -1,5 +1,5 @@
 /* auth.c: Authorization management routines for gzochid
- * Copyright (C) 2011 Julian Graham
+ * Copyright (C) 2013 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -16,7 +16,9 @@
  */
 
 #include <glib.h>
+#include <stddef.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "app.h"
 #include "auth.h"
@@ -60,3 +62,13 @@ void *gzochid_auth_identity_deserializer
   g_string_erase (in, 0, strlen (name) + 1);
   return identity;
 }
+
+void gzochid_auth_identity_finalizer
+(gzochid_application_context *context, void *ptr)
+{
+  gzochid_auth_identity *identity = (gzochid_auth_identity *) ptr;
+
+  free (identity->name);
+
+  free (identity);
+}
