@@ -1,5 +1,5 @@
 /* config.c: Configuration management routines for gzochid
- * Copyright (C) 2012 Julian Graham
+ * Copyright (C) 2013 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #include <libgen.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <strings.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -29,7 +30,7 @@
 #include "config.h"
 #include "log.h"
 
-int gzochid_config_to_boolean (char *str, int def)
+gboolean gzochid_config_to_boolean (char *str, gboolean def)
 {
   if (str == NULL)
     return def;
@@ -50,6 +51,16 @@ int gzochid_config_to_int (char *str, int def)
   return def;
 }
 
+long gzochid_config_to_long (char *str, long def)
+{
+  long ret = 0;
+  if (str == NULL)
+    return def;
+  if (sscanf (str, "%ld", &ret) == 1)
+    return ret;
+  return def;
+}
+
 GHashTable *gzochid_config_keyfile_extract_config 
 (GKeyFile *key_file, char *group)
 {
