@@ -1,5 +1,5 @@
 /* data.c: Application data management routines for gzochid
- * Copyright (C) 2012 Julian Graham
+ * Copyright (C) 2013 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -49,17 +49,27 @@ void gzochid_oid_holder_free (gzochid_oid_holder *holder)
 }
 
 static gzochid_data_transaction_context *create_transaction_context 
-(gzochid_application_context *app_context)
+(gzochid_application_context *app_context, struct timeval *timeout)
 {
   gzochid_data_transaction_context *tx_context = 
     calloc (1, sizeof (gzochid_data_transaction_context));
 
   tx_context->context = app_context;
 
-  tx_context->oids_transaction = 
-    gzochid_storage_transaction_begin (app_context->oids);
-  tx_context->names_transaction =
-    gzochid_storage_transaction_begin (app_context->names);
+  if (timeout != NULL)
+    {
+      tx_context->oids_transaction = 
+	gzochid_storage_transaction_begin_timed (app_context->oids, *timeout);
+      tx_context->names_transaction =
+	gzochid_storage_transaction_begin_timed (app_context->names, *timeout);
+    }
+  else
+    {
+      tx_context->oids_transaction = 
+	gzochid_storage_transaction_begin (app_context->oids);
+      tx_context->names_transaction =
+	gzochid_storage_transaction_begin (app_context->names);
+    }
 
   tx_context->oids_to_references = g_hash_table_new (g_str_hash, g_str_equal);
   tx_context->ptrs_to_references = g_hash_table_new 
@@ -90,13 +100,10 @@ static void transaction_context_free (gzochid_data_transaction_context *context)
   free (context);
 }
 
-static void flush_reference (gpointer data, gpointer user_data)
+static gboolean flush_reference 
+(gzochid_data_managed_reference *reference,
+ gzochid_data_transaction_context *context)
 {
-  gzochid_data_transaction_context *context = 
-    (gzochid_data_transaction_context *) user_data;
-  gzochid_data_managed_reference *reference = 
-    (gzochid_data_managed_reference *) data;
-
   char *oid_str = NULL;
   GString *out = NULL;
 
@@ -117,7 +124,14 @@ static void flush_reference (gpointer data, gpointer user_data)
 
       reference->serialization->serializer
 	(context->context, reference->obj, out);
-      gzochid_storage_transaction_put 
+
+      if (gzochid_transaction_rollback_only ())
+	{
+	  g_string_free (out, FALSE);	  
+	  return FALSE;
+	}
+
+      gzochid_storage_transaction_put
 	(context->oids_transaction, oid_str, strlen (oid_str) + 1, 
 	 out->str, out->len);
 
@@ -129,6 +143,8 @@ static void flush_reference (gpointer data, gpointer user_data)
     default:
       assert (1 == 0);
     }
+
+  return TRUE;
 }
 
 static int data_prepare (gpointer data)
@@ -136,24 +152,59 @@ static int data_prepare (gpointer data)
   gzochid_data_transaction_context *context = 
     (gzochid_data_transaction_context *) data;
   GList *references = g_hash_table_get_values (context->oids_to_references);
-
+  GList *reference_ptr = references;
+  gboolean flush_failed = FALSE;
+  
   gzochid_storage_lock (context->context->oids);
   gzochid_storage_lock (context->context->names);
 
-  g_list_foreach (references, flush_reference, context);
+  while (reference_ptr != NULL)
+    {
+      if (!flush_reference (reference_ptr->data, context))
+	{
+	  flush_failed = TRUE;
+	  break;
+	}
+
+      reference_ptr = reference_ptr->next;
+    }
   g_list_free (references);
 
-  gzochid_storage_transaction_check (context->oids_transaction);
+  if (flush_failed)
+    return FALSE;
+
+  gzochid_storage_transaction_prepare (context->oids_transaction);
   if (context->oids_transaction->rollback)
     return FALSE;
 
-  gzochid_storage_transaction_check (context->names_transaction);
+  gzochid_storage_transaction_prepare (context->names_transaction);
   if (context->oids_transaction->rollback)
     return FALSE;
 
   return TRUE;
 }
 
+static void finalize_references (gzochid_data_transaction_context *context)
+{
+  GList *references = g_hash_table_get_values (context->oids_to_references);
+  GList *reference_ptr = references;
+
+  while (reference_ptr != NULL)
+    {
+      gzochid_data_managed_reference *reference =
+	(gzochid_data_managed_reference *) reference_ptr->data;
+
+      if (reference->obj != NULL)
+	{
+	  reference->serialization->finalizer 
+	    (context->context, reference->obj);
+	  reference->obj = NULL;
+	}
+      reference_ptr = reference_ptr->next;
+    }
+  g_list_free (references);
+}
+
 static void data_commit (gpointer data)
 {
   gzochid_data_transaction_context *context = 
@@ -162,6 +213,8 @@ static void data_commit (gpointer data)
   gzochid_storage_transaction_commit (context->oids_transaction);
   gzochid_storage_transaction_commit (context->names_transaction);
 
+  finalize_references (context);
+
   gzochid_storage_unlock (context->context->names);
   gzochid_storage_unlock (context->context->oids);
 
@@ -176,6 +229,8 @@ static void data_rollback (gpointer data)
   gzochid_storage_transaction_rollback (context->oids_transaction);
   gzochid_storage_transaction_rollback (context->names_transaction);
 
+  finalize_references (context);
+
   gzochid_storage_unlock (context->context->names);
   gzochid_storage_unlock (context->context->oids);
 
@@ -251,7 +306,8 @@ static void get_binding
   char *oid_str = gzochid_storage_transaction_get 
     (context->names_transaction, name, strlen (name) + 1, NULL);
   if (context->names_transaction->rollback)
-    gzochid_transaction_mark_for_rollback (&data_participant);
+    gzochid_transaction_mark_for_rollback 
+      (&data_participant, context->names_transaction->should_retry);
 
   if (oid_str != NULL)
     {
@@ -270,7 +326,8 @@ static void set_binding
      strlen (oid_str) + 1);
 
   if (context->names_transaction->rollback)
-    gzochid_transaction_mark_for_rollback (&data_participant);
+    gzochid_transaction_mark_for_rollback 
+      (&data_participant, context->names_transaction->should_retry);
 
   free (oid_str);
 }
@@ -400,7 +457,8 @@ void dereference
   data = gzochid_storage_transaction_get
     (context->oids_transaction, oid_str, strlen (oid_str) + 1, &data_len);
   if (context->oids_transaction->rollback)
-    gzochid_transaction_mark_for_rollback (&data_participant);
+    gzochid_transaction_mark_for_rollback 
+      (&data_participant, context->oids_transaction->should_retry);
 
   if (data == NULL)
     {
@@ -458,8 +516,16 @@ static void join_transaction (gzochid_application_context *context)
 {
   if (!gzochid_transaction_active ()
       || gzochid_transaction_context (&data_participant) == NULL)
-    gzochid_transaction_join
-      (&data_participant, create_transaction_context (context));
+    {
+      gzochid_data_transaction_context *tx_context = NULL;
+      if (gzochid_transaction_timed ())
+	{
+	  struct timeval remaining = gzochid_transaction_time_remaining ();
+	  tx_context = create_transaction_context (context, &remaining);
+	}
+      else tx_context = create_transaction_context (context, NULL); 
+      gzochid_transaction_join (&data_participant, tx_context);
+    }
 }
 
 void *gzochid_data_get_binding
@@ -510,14 +576,16 @@ char *gzochid_data_next_binding_oid
   next_key = gzochid_storage_transaction_next_key 
     (tx_context->names_transaction, key, strlen (key) + 1, NULL);
   if (tx_context->names_transaction->rollback)
-    gzochid_transaction_mark_for_rollback (&data_participant);
+    gzochid_transaction_mark_for_rollback 
+      (&data_participant, tx_context->names_transaction->should_retry);
 
   if (next_key != NULL)
     {
       char *next_value = gzochid_storage_transaction_get 
 	(tx_context->names_transaction, next_key, strlen (next_key) + 1, NULL);
       if (tx_context->names_transaction->rollback)
-	gzochid_transaction_mark_for_rollback (&data_participant);
+	gzochid_transaction_mark_for_rollback 
+	  (&data_participant, tx_context->names_transaction->should_retry);
 
       mpz_set_str (oid, next_value, 16);
       free (next_value);
@@ -559,7 +627,8 @@ void gzochid_data_remove_binding
   gzochid_storage_transaction_delete 
     (tx_context->names_transaction, name, strlen (name) + 1);
   if (tx_context->names_transaction->rollback)
-    gzochid_transaction_mark_for_rollback (&data_participant);
+    gzochid_transaction_mark_for_rollback 
+      (&data_participant, tx_context->names_transaction->should_retry);
 
   free (oid_str);
 }
@@ -708,7 +777,8 @@ void gzochid_data_mark
       data = gzochid_storage_transaction_get_for_update
 	(tx_context->oids_transaction, oid_str, strlen (oid_str) + 1, NULL);
       if (tx_context->oids_transaction->rollback)
-	gzochid_transaction_mark_for_rollback (&data_participant);
+	gzochid_transaction_mark_for_rollback 
+	  (&data_participant, tx_context->oids_transaction->should_retry);
 
       free (oid_str);
       free (data);
@@ -738,16 +808,6 @@ static void init_persistence_task
   task->holder = holder;
 } 
 
-static gzochid_persistence_task_data *gzochid_persistence_task_data_new
-(gzochid_io_serialization *serialization, gpointer data,
- gzochid_oid_holder *holder)
-{
-  gzochid_persistence_task_data *task = 
-    malloc (sizeof (gzochid_persistence_task_data));
-  init_persistence_task (task, serialization, data, holder);
-  return task;
-}
-
 static gzochid_prefix_binding_persistence_task_data *
 gzochid_prefix_binding_persistence_task_data_new
 (gzochid_io_serialization *serialization, gpointer data,
@@ -805,38 +865,6 @@ static void prefix_binding_persistence_task_worker
   g_string_free (binding, FALSE);
 }
 
-gzochid_task *gzochid_data_persistence_task_new 
-(gzochid_application_context *context, gzochid_auth_identity *identity,
- gzochid_io_serialization *serialization, gpointer data,
- gzochid_oid_holder *holder)
-{
-  gzochid_persistence_task_data *task_data =
-    gzochid_persistence_task_data_new (serialization, data, holder);
-  gzochid_transactional_application_task *transactional_task = 
-    gzochid_transactional_application_task_new 
-    (persistence_task_worker, task_data);
-  gzochid_application_task *application_task = gzochid_application_task_new 
-    (context, identity, gzochid_application_transactional_task_worker,
-     transactional_task);
-  return gzochid_task_immediate_new 
-    (gzochid_application_task_thread_worker, application_task);
-}
-
-void gzochid_data_persistence_task_free (gzochid_task *task)
-{
-  gzochid_application_task *application_task = 
-    (gzochid_application_task *) task->data;
-  gzochid_transactional_application_task *transactional_task = 
-    (gzochid_transactional_application_task *) application_task->data;
-  gzochid_persistence_task_data *data = 
-    (gzochid_persistence_task_data *) transactional_task->data;
-
-  free (data);
-  free (transactional_task);
-  free (application_task);
-  free (task);
-}
-
 gzochid_task *gzochid_data_prefix_binding_persistence_task_new
 (gzochid_application_context *context, gzochid_auth_identity *identity, 
  gzochid_io_serialization *serialization, gpointer data, 
@@ -845,17 +873,33 @@ gzochid_task *gzochid_data_prefix_binding_persistence_task_new
   gzochid_prefix_binding_persistence_task_data *task_data =
     gzochid_prefix_binding_persistence_task_data_new 
     (serialization, data, holder, prefix);
-  gzochid_transactional_application_task *transactional_task = 
-    gzochid_transactional_application_task_new 
-    (prefix_binding_persistence_task_worker, task_data);
+  gzochid_application_task *transactional_task = 
+    gzochid_application_task_new 
+    (context, identity, prefix_binding_persistence_task_worker, task_data);
+  gzochid_transactional_application_task_execution *execution = 
+    gzochid_transactional_application_task_execution_new (transactional_task);
   gzochid_application_task *application_task = gzochid_application_task_new 
-    (context, identity, gzochid_application_transactional_task_worker,
-     transactional_task);
+    (context, identity, 
+     gzochid_application_resubmitting_transactional_task_worker, execution);
+
   return gzochid_task_immediate_new 
     (gzochid_application_task_thread_worker, application_task);
 }
 
 void gzochid_data_prefix_binding_persistence_task_free (gzochid_task *task)
 {
-  gzochid_data_persistence_task_free (task);
+  gzochid_application_task *application_task = 
+    (gzochid_application_task *) task->data;
+  gzochid_transactional_application_task_execution *execution = 
+    (gzochid_transactional_application_task_execution *) 
+    application_task->data;
+  gzochid_application_task *transactional_task = execution->task;
+  gzochid_persistence_task_data *data = 
+    (gzochid_persistence_task_data *) transactional_task->data;
+  
+  free (data);
+  free (transactional_task);
+  gzochid_transactional_application_task_execution_free (execution);
+  free (application_task);
+  free (task);
 }
