@@ -1,5 +1,5 @@
 /* game.c: Game context management routines for gzochid
- * Copyright (C) 2012 Julian Graham
+ * Copyright (C) 2013 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -38,6 +38,8 @@
 
 #define GAME_DESCRIPTOR_XML "game.xml"
 
+#define DEFAULT_TX_TIMEOUT_MS 100
+
 static void scan_app_dir (gzochid_game_context *context, char *dir)
 {
   char *descriptor_file = g_strconcat (dir, "/", GAME_DESCRIPTOR_XML, NULL);
@@ -128,6 +130,12 @@ static void initialize_serialization_registry
   gzochid_task_initialize_serialization_registry ();
 }
 
+static void initialize_client_received_message_task_serialization
+(int from_state, int to_state, gpointer user_data)
+{
+  gzochid_register_client_received_message_task_serialization ();
+}
+
 static void initialize_complete
 (int from_state, int to_state, gpointer user_data)
 {
@@ -163,6 +171,7 @@ gzochid_task_queue *gzochid_task_queue_new (GThreadPool *pool)
 void gzochid_game_context_init
 (gzochid_game_context *context, gzochid_context *parent, GHashTable *config)
 {
+  long tx_timeout_ms = 0;
   gzochid_fsm *fsm = gzochid_fsm_new 
     ("game", GZOCHID_GAME_STATE_INITIALIZING, "INITIALIZING");
 
@@ -183,6 +192,11 @@ void gzochid_game_context_init
   context->apps_dir = g_hash_table_lookup (config, "server.fs.apps");
   context->work_dir = g_hash_table_lookup (config, "server.fs.data");
 
+  tx_timeout_ms = gzochid_config_to_long 
+    (g_hash_table_lookup (config, "tx.timeout"), DEFAULT_TX_TIMEOUT_MS);
+  context->tx_timeout.tv_sec = tx_timeout_ms / 1000;
+  context->tx_timeout.tv_usec = (tx_timeout_ms % 1000) * 1000;
+
   gzochid_fsm_add_transition 
     (fsm, GZOCHID_GAME_STATE_INITIALIZING, GZOCHID_GAME_STATE_RUNNING);
   gzochid_fsm_add_transition
@@ -190,13 +204,16 @@ void gzochid_game_context_init
 
   gzochid_fsm_on_enter 
     (fsm, GZOCHID_GAME_STATE_INITIALIZING, initialize_server, context);
-  gzochid_fsm_on_enter 
-    (fsm, GZOCHID_GAME_STATE_INITIALIZING, initialize_scheme_api, context);
-  gzochid_fsm_on_enter 
-    (fsm, GZOCHID_GAME_STATE_INITIALIZING, initialize_apps, context);
   gzochid_fsm_on_enter 
     (fsm, GZOCHID_GAME_STATE_INITIALIZING, initialize_serialization_registry, 
      context);
+  gzochid_fsm_on_enter 
+    (fsm, GZOCHID_GAME_STATE_INITIALIZING, initialize_scheme_api, context);
+  gzochid_fsm_on_enter 
+    (fsm, GZOCHID_GAME_STATE_INITIALIZING, 
+     initialize_client_received_message_task_serialization, context);
+  gzochid_fsm_on_enter 
+    (fsm, GZOCHID_GAME_STATE_INITIALIZING, initialize_apps, context);
   gzochid_fsm_on_enter 
     (fsm, GZOCHID_GAME_STATE_INITIALIZING, initialize_complete, context);
   
