@@ -1,5 +1,5 @@
 /* scheme.c: Supplementary Scheme routines for gzochid
- * Copyright (C) 2012 Julian Graham
+ * Copyright (C) 2013 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@
 #include "session.h"
 #include "task.h"
 #include "tx.h"
+#include "txlog.h"
 
 #include "api/channel.h"
 #include "api/data.h"
@@ -75,6 +76,7 @@ static SCM scm_r6rs_raise;
 static SCM scm_make_object_removed_condition;
 static SCM scm_make_name_exists_condition;
 static SCM scm_make_name_not_bound_condition;
+static SCM scm_transaction_retry_condition_p;
 
 static SCM resolve_procedure (char *procedure, GList *module)
 {
@@ -95,71 +97,91 @@ static gpointer scheme_invoke_inner (gpointer data)
   return gzochid_guile_invoke (procedure, args, exception_var);
 }
 
-SCM gzochid_scheme_invoke
-(gzochid_application_context *context, gzochid_auth_identity *identity, 
- char *procedure, GList *module, SCM args, SCM exception_var)
+static SCM scheme_invoke 
+(gzochid_application_context *context, gzochid_auth_identity *identity,
+ SCM proc, SCM args, SCM exception_var)
 {
   void *data[3];
   SCM ret = SCM_EOL;
-  SCM load_path = SCM_EOL, new_load_path = SCM_EOL, backup_load_path = SCM_EOL;
-  GList *path_ptr = context->descriptor->load_paths;
 
-  load_path = scm_module_variable 
-    (scm_c_resolve_module ("guile"), scm_from_locale_symbol ("%load-path"));
-  backup_load_path = scm_variable_ref (load_path);
-  new_load_path = backup_load_path;
-
-  while (path_ptr != NULL)
-    {
-      new_load_path = scm_cons 
-	(scm_from_locale_string ((char *) path_ptr->data), new_load_path);      
-      path_ptr = path_ptr->prev;
-    }
-  
-  scm_variable_set_x (load_path, new_load_path);
-
-  data[0] = resolve_procedure (procedure, module);
+  data[0] = proc;
   data[1] = args;
   data[2] = exception_var;
 
   ret = (SCM) gzochid_with_application_context 
     (context, identity, scheme_invoke_inner, data);
-  scm_variable_set_x (load_path, backup_load_path);
 
+  scm_remember_upto_here_1 (proc);
   scm_remember_upto_here_1 (args);
   scm_remember_upto_here_1 (exception_var);
 
   return ret;
 }
 
-static int scheme_prepare (gpointer data) { return FALSE; }
-static void scheme_commit (gpointer data) { assert (1 == 0); }
+SCM gzochid_scheme_invoke
+(gzochid_application_context *context, gzochid_auth_identity *identity, 
+ char *procedure, GList *module, SCM args, SCM exception_var)
+{
+  return scheme_invoke 
+    (context, identity, resolve_procedure (procedure, module), args, 
+     exception_var);
+}
+
+static int scheme_prepare (gpointer data) { return TRUE; }
+static void scheme_commit (gpointer data) { }
 static void scheme_rollback (gpointer data) { }
 
 static gzochid_transaction_participant scheme_participant = 
   { "scheme", scheme_prepare, scheme_commit, scheme_rollback };
 
+static gboolean is_transaction_retry (SCM cond)
+{
+  return scm_is_true (scm_call_1 (scm_transaction_retry_condition_p, cond));
+}
+
 void gzochid_scheme_application_worker 
+(gzochid_application_context *context, gzochid_auth_identity *identity, 
+ gpointer ptr)
+{
+  void **data = (void **) ptr;
+
+  SCM proc = (SCM) data[0];
+  SCM args = (SCM) data[1];
+  SCM exception_var = (SCM) data[2];
+  SCM *ret = (SCM *) data[3];
+
+  gzochid_transaction_join (&scheme_participant, NULL);
+
+  *ret = scheme_invoke (context, identity, proc, args, exception_var);
+
+  if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
+    gzochid_transaction_mark_for_rollback 
+      (&scheme_participant, 
+       is_transaction_retry (scm_variable_ref (exception_var)));
+}
+
+void gzochid_scheme_application_task_worker 
 (gzochid_application_context *context, gzochid_auth_identity *identity, 
  gpointer task)
 {
   SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
 
+  gzochid_transaction_join (&scheme_participant, NULL);
+
   gzochid_scheme_invoke 
     (context, 
      identity,
      "gzochi:run-task", 
-     g_list_append 
+     g_list_append
      (g_list_append 
       (g_list_append (NULL, "gzochi"), "private"), "task"), 
      scm_list_1 ((SCM) task), 
      exception_var);
 
   if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
-    {
-      gzochid_transaction_join (&scheme_participant, NULL);
-      gzochid_transaction_mark_for_rollback (&scheme_participant);
-    }
+    gzochid_transaction_mark_for_rollback 
+      (&scheme_participant,
+       is_transaction_retry (scm_variable_ref (exception_var)));
 }
 
 static gzochid_application_callback *scm_to_callback 
@@ -192,6 +214,8 @@ void gzochid_scheme_application_initialized_worker
   callback_reference = gzochid_data_create_reference 
     (context, &gzochid_scheme_data_serialization, callback);
 
+  gzochid_transaction_join (&scheme_participant, NULL);
+
   gzochid_scheme_invoke 
     (context,
      identity,
@@ -209,10 +233,9 @@ void gzochid_scheme_application_initialized_worker
      exception_var);
 
   if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
-    {
-      gzochid_transaction_join (&scheme_participant, NULL);
-      gzochid_transaction_mark_for_rollback (&scheme_participant);
-    }
+    gzochid_transaction_mark_for_rollback 
+      (&scheme_participant,
+       is_transaction_retry (scm_variable_ref (exception_var)));
   else gzochid_data_set_binding_to_oid 
 	 (context, "s.initializer", callback_reference->oid);
 }
@@ -220,15 +243,20 @@ void gzochid_scheme_application_initialized_worker
 static gpointer unpack_handler (gpointer data)
 {
   SCM handler = (SCM) data;
+  SCM received_message = gzochid_scheme_handler_received_message (handler);
+  SCM disconnected = gzochid_scheme_handler_disconnected (handler);
+
   gzochid_application_context *context = 
     gzochid_get_current_application_context ();
   gzochid_client_session_handler *session_handler = 
     malloc (sizeof (gzochid_client_session_handler));
 
+  scm_gc_protect_object (received_message);
+  scm_gc_protect_object (disconnected);
+
   session_handler->received_message = scm_to_callback 
-    (context, gzochid_scheme_handler_received_message (handler));
-  session_handler->disconnected = scm_to_callback 
-    (context, gzochid_scheme_handler_disconnected (handler));
+    (context, received_message);
+  session_handler->disconnected = scm_to_callback (context, disconnected);
 
   return session_handler;
 }
@@ -270,6 +298,8 @@ void gzochid_scheme_application_logged_in_worker
   callback_reference = gzochid_data_create_reference 
     (context, &gzochid_scheme_data_serialization, cb);
 
+  gzochid_transaction_join (&scheme_participant, NULL);
+
   handler = gzochid_scheme_invoke 
     (context,
      identity,
@@ -282,12 +312,14 @@ void gzochid_scheme_application_logged_in_worker
      exception_var);
   
   if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
-    {
-      gzochid_transaction_join (&scheme_participant, NULL);
-      gzochid_transaction_mark_for_rollback (&scheme_participant);
-    }
+    gzochid_transaction_mark_for_rollback 
+      (&scheme_participant, 
+       is_transaction_retry (scm_variable_ref (exception_var)));
   else if (scm_is_false (handler))
-    gzochid_client_session_send_login_failure (context, session);
+    {
+      gzochid_client_session_send_login_failure (context, session);
+      gzochid_client_session_disconnect (context, session);
+    }
   else
     {
       session->handler = (gzochid_client_session_handler *) 
@@ -346,7 +378,9 @@ void gzochid_scheme_application_received_message_worker
   if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
     {
       gzochid_transaction_join (&scheme_participant, NULL);
-      gzochid_transaction_mark_for_rollback (&scheme_participant);
+      gzochid_transaction_mark_for_rollback 
+	(&scheme_participant, 
+	 is_transaction_retry (scm_variable_ref (exception_var)));
     }
 
   g_list_free (gpa);
@@ -370,9 +404,23 @@ void gzochid_scheme_application_disconnected_worker
 
   session_reference = gzochid_data_create_reference_to_oid 
     (context, &gzochid_client_session_serialization, session_oid);
+  mpz_clear (session_oid);
   gzochid_data_dereference (session_reference);
   session = (gzochid_client_session *) session_reference->obj;
 
+  /* A client may disconnect before the login process has completed or after
+     it has failed enough times to stop being retried. */
+
+  if (session->handler == NULL)
+    {
+      gzochid_tx_warning 
+	(context, "Session '%s' disconnected after incomplete login.", ptr);
+
+      g_list_free (gpa);
+      gzochid_data_remove_object (session_reference);
+      return;
+    }
+
   callback_reference =
     gzochid_data_create_reference_to_oid
     (context, &gzochid_scheme_data_serialization, 
@@ -389,7 +437,9 @@ void gzochid_scheme_application_disconnected_worker
   if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
     {
       gzochid_transaction_join (&scheme_participant, NULL);
-      gzochid_transaction_mark_for_rollback (&scheme_participant);
+      gzochid_transaction_mark_for_rollback 
+	(&scheme_participant, 
+	 is_transaction_retry (scm_variable_ref (exception_var)));
     }
   else gzochid_data_remove_object (session_reference);
 
@@ -405,13 +455,12 @@ static void scheme_worker_serializer
 static gzochid_application_worker scheme_worker_deserializer
 (gzochid_application_context *context, GString *in)
 {
-  return gzochid_scheme_application_worker;
+  return gzochid_scheme_application_task_worker;
 }
 
 gboolean is_managed_record (SCM obj)
 {
-  SCM ret = scm_call_1 (scm_managed_record_p, obj);
-  return ret == SCM_BOOL_T ? TRUE : FALSE;
+  return scm_is_true (scm_call_1 (scm_managed_record_p, obj));
 }
 
 gboolean for_all (SCM lst, gboolean (*pred) (SCM))
@@ -434,6 +483,7 @@ static void scheme_managed_record_serializer
   GList *gpd = g_list_append 
     (g_list_append (g_list_append (NULL, "gzochi"), "private"), "data");
   SCM vec = SCM_EOL, port = SCM_EOL, proc = SCM_EOL;
+  SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
   SCM port_and_proc = 
     scm_struct_ref (scm_open_bytevector_output_port (SCM_BOOL_F), SCM_INUM0);
   int vec_len = 0;
@@ -446,7 +496,19 @@ static void scheme_managed_record_serializer
      NULL,
      "gzochi:serialize-managed-record", gpd,
      scm_list_2 (port, arg), 
-     SCM_BOOL_F);
+     exception_var);
+
+  if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
+    {
+      /* Don't join the transaction here as the serializer should only be
+	 called during the PREPARING phase, which is too late to join. */
+
+      gzochid_transaction_mark_for_rollback 
+	(&scheme_participant, 
+	 is_transaction_retry (scm_variable_ref (exception_var)));
+      g_list_free (gpd);
+      return;
+    }
 
   vec = scm_call_0 (proc);
 
@@ -465,6 +527,7 @@ static void *scheme_managed_record_deserializer
 {
   int vec_len = gzochi_common_io_read_int ((unsigned char *) in->str, 0);
   SCM vec = SCM_EOL, port = SCM_EOL, record = SCM_EOL;
+  SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
   
   g_string_erase (in, 0, 4);
   vec = scm_take_u8vector ((unsigned char *) in->str, vec_len);
@@ -479,10 +542,18 @@ static void *scheme_managed_record_deserializer
      (g_list_append 
       (g_list_append (NULL, "gzochi"), "private"), "data"),
      scm_list_1 (port),
-     SCM_BOOL_F);
+     exception_var);
 
-  scm_gc_protect_object (record);
   g_string_erase (in, 0, vec_len);
+  
+  /* Don't join the transaction here as the serializer should only be
+     called during the PREPARING phase, which is too late to join. */
+  
+  if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
+    gzochid_transaction_mark_for_rollback 
+      (&scheme_participant, 
+       is_transaction_retry (scm_variable_ref (exception_var)));
+  else scm_gc_protect_object (record);
 
   return record;
 }
@@ -531,11 +602,23 @@ static void *scheme_deserializer
   return scm_with_guile (scheme_deserializer_inner, args);
 }
 
+static void *scheme_finalizer_inner (void *data)
+{
+  scm_gc_unprotect_object ((SCM) data);
+  return NULL;
+}
+
+static void scheme_finalizer 
+(gzochid_application_context *context, gpointer data)
+{
+  scm_with_guile (scheme_finalizer_inner, data);
+}
+
 static gzochid_application_worker_serialization scheme_worker_serialization =
   { scheme_worker_serializer, scheme_worker_deserializer };
 
 gzochid_io_serialization gzochid_scheme_data_serialization =
-  { scheme_serializer, scheme_deserializer };
+  { scheme_serializer, scheme_deserializer, scheme_finalizer };
 
 gzochid_application_task_serialization gzochid_scheme_task_serialization = 
   { 
@@ -544,16 +627,6 @@ gzochid_application_task_serialization gzochid_scheme_task_serialization =
     &gzochid_scheme_data_serialization 
   };
 
-void *init_scheme_task (gpointer ptr)
-{
-  SCM lst = (SCM) ptr;
-  
-  return scm_apply_2 (scm_make_callback, 
-		      scm_list_ref (lst, SCM_INUM0),
-		      scm_list_ref (lst, SCM_INUM1),
-		      scm_list_ref (lst, scm_from_short (2)));
-}
-
 gzochid_application_task *gzochid_scheme_task_new 
 (gzochid_application_context *context, gzochid_auth_identity *identity,
  char *procedure, GList *module_name, SCM data)
@@ -570,10 +643,12 @@ gzochid_application_task *gzochid_scheme_task_new
 		data)), 
      SCM_BOOL_F);
 
-  gzochid_application_task *task = gzochid_application_task_new
-    (context, identity, gzochid_scheme_application_worker, scm_data);
+  gzochid_application_task *task = NULL; 
+
+  scm_gc_protect_object (scm_data);
+  task = gzochid_application_task_new
+    (context, identity, gzochid_scheme_application_task_worker, scm_data);
   
-  scm_gc_protect_object (task->data);
   return task;
 }
 
@@ -646,7 +721,6 @@ GHashTable *gzochid_scheme_hashtable_to_ghashtable
 
       key_list = SCM_CDR (key_list);
     }
-  
 
   return ret;
 }
@@ -801,97 +875,72 @@ SCM gzochid_scheme_create_channel (gzochid_channel *channel, mpz_t oid)
   return ret;
 }
 
-static void initialize_binding (SCM module, SCM *binding, char *name)
+static void bind_scm (char *module, SCM *binding, char *name)
 {
-  SCM var = scm_c_module_lookup (module, name);
+  SCM var = scm_c_public_variable (module, name);
+
+  if (scm_is_false (var))
+    g_error ("Missing Scheme binding for `%s'. Aborting...", name);
+
   *binding = scm_variable_ref (var);
   scm_gc_protect_object (*binding);
 }
 
 static void *initialize_bindings (void *ptr)
 {
-  SCM rnrs_exceptions = scm_c_resolve_module ("rnrs exceptions");
-  SCM rnrs_hashtables = scm_c_resolve_module ("rnrs hashtables");
+  bind_scm ("rnrs exceptions", &scm_r6rs_raise, "raise");
 
-  SCM gzochi_app = scm_c_resolve_module ("gzochi app");
-  SCM gzochi_channel = scm_c_resolve_module ("gzochi channel");
-  SCM gzochi_client = scm_c_resolve_module ("gzochi client");
-  SCM gzochi_conditions = scm_c_resolve_module ("gzochi conditions");
-  SCM gzochi_data = scm_c_resolve_module ("gzochi data");
+  bind_scm ("rnrs hashtables", &gzochid_scheme_string_hash, "string-hash");
+  bind_scm ("rnrs base", &gzochid_scheme_string_equiv, "equal?");
 
-  SCM gzochi_private_client = scm_c_resolve_module ("gzochi private client");
-  SCM gzochi_private_data = scm_c_resolve_module ("gzochi private data");
-  SCM gzochi_private_task = scm_c_resolve_module ("gzochi private task");
+  bind_scm ("rnrs hashtables", &scm_hashtable_keys, "hashtable-keys");
+  bind_scm ("rnrs hashtables", &scm_hashtable_ref, "hashtable-ref");
+  bind_scm ("rnrs hashtables", &scm_hashtable_set_x, "hashtable-set!");
+  bind_scm ("rnrs hashtables", &scm_make_hashtable, "make-hashtable");
 
-  gzochid_scheme_scm_module_gzochi_private_app = 
-    scm_c_resolve_module ("gzochi private app");
-  scm_gc_protect_object (gzochid_scheme_scm_module_gzochi_private_app);
-  
-  initialize_binding (rnrs_exceptions, &scm_r6rs_raise, "raise");
+  bind_scm ("gzochi app", &scm_make_callback, "gzochi:make-callback");
+  bind_scm ("gzochi app", &scm_callback_module, "gzochi:callback-module");
+  bind_scm ("gzochi app", &scm_callback_procedure, "gzochi:callback-procedure");
+  bind_scm ("gzochi app", &scm_callback_data, "gzochi:callback-data");
 
-  initialize_binding 
-    (rnrs_hashtables, &gzochid_scheme_string_hash, "string-hash");
-  initialize_binding (rnrs_hashtables, &gzochid_scheme_string_equiv, "equal?");
+  bind_scm ("gzochi private channel", &scm_make_channel, 
+	    "gzochi:make-channel");
+  bind_scm ("gzochi private channel", &scm_channel_oid, "gzochi:channel-oid");
 
-  initialize_binding (rnrs_hashtables, &scm_hashtable_keys, "hashtable-keys");
-  initialize_binding (rnrs_hashtables, &scm_hashtable_ref, "hashtable-ref");
-  initialize_binding (rnrs_hashtables, &scm_hashtable_set_x, "hashtable-set!");
-  initialize_binding (rnrs_hashtables, &scm_make_hashtable, "make-hashtable");
+  bind_scm ("gzochi client", &scm_handler_received_message,
+	    "gzochi:client-session-listener-received-message");
+  bind_scm ("gzochi client", &scm_handler_disconnected,
+	    "gzochi:client-session-listener-disconnected");
 
-  initialize_binding (gzochi_app, &scm_make_callback, "gzochi:make-callback");
-  initialize_binding 
-    (gzochi_app, &scm_callback_module, "gzochi:callback-module");
-  initialize_binding
-    (gzochi_app, &scm_callback_procedure, "gzochi:callback-procedure");
-  initialize_binding (gzochi_app, &scm_callback_data, "gzochi:callback-data");
+  bind_scm ("gzochi conditions", &scm_make_name_exists_condition,
+	    "gzochi:make-name-exists-condition");
+  bind_scm ("gzochi conditions", &scm_make_name_not_bound_condition,
+	    "gzochi:make-name-not-bound-condition");
+  bind_scm ("gzochi conditions", &scm_make_object_removed_condition,
+	    "gzochi:make-object-removed-condition");
+  bind_scm ("gzochi conditions", &scm_transaction_retry_condition_p,
+	    "gzochi:transaction-retry-condition?");
 
-  initialize_binding (gzochi_channel, &scm_make_channel, "gzochi:make-channel");
-  initialize_binding (gzochi_channel, &scm_channel_oid, "gzochi:channel-oid");
+  bind_scm ("gzochi data", &scm_make_managed_hashtable, 
+	    "gzochi:make-managed-hashtable");
+  bind_scm ("gzochi data", &scm_managed_record_p, "gzochi:managed-record?");
 
-  initialize_binding
-    (gzochi_client, &scm_handler_received_message,
-     "gzochi:client-session-listener-received-message");
-  initialize_binding
-    (gzochi_client, &scm_handler_disconnected,
-     "gzochi:client-session-listener-disconnected");
+  bind_scm ("gzochi private data", &scm_managed_record_serialize, 
+	    "gzochi:serialize-managed-record");
+  bind_scm ("gzochi private data", &scm_managed_record_deserialize,
+	    "gzochi:deserialize-managed-record");
+  bind_scm ("gzochi private data", &scm_managed_reference_oid, 
+	    "gzochi:managed-reference-oid");
 
-  initialize_binding
-    (gzochi_conditions, &scm_make_name_exists_condition,
-     "gzochi:make-name-exists-condition");
-  initialize_binding
-    (gzochi_conditions, &scm_make_name_not_bound_condition,
-     "gzochi:make-name-not-bound-condition");
-  initialize_binding
-    (gzochi_conditions, &scm_make_object_removed_condition,
-     "gzochi:make-object-removed-condition");
+  bind_scm ("gzochi private client", &scm_make_client_session, 
+	    "gzochi:make-client-session");
+  bind_scm ("gzochi private client", &scm_client_session_oid, 
+	    "gzochi:client-session-oid");
 
-  initialize_binding
-    (gzochi_data, &scm_make_managed_hashtable, "gzochi:make-managed-hashtable");
-  initialize_binding 
-    (gzochi_data, &scm_managed_record_p, "gzochi:managed-record?");
-
-  initialize_binding 
-    (gzochi_private_data, &scm_managed_record_serialize, 
-     "gzochi:serialize-managed-record");
-  initialize_binding 
-    (gzochi_private_data, &scm_managed_record_deserialize,
-     "gzochi:deserialize-managed-record");
-  initialize_binding
-    (gzochi_private_data, &scm_managed_reference_oid, 
-     "gzochi:managed-reference-oid");
-
-  initialize_binding
-    (gzochi_private_client, &scm_make_client_session, 
-     "gzochi:make-client-session");
-  initialize_binding
-    (gzochi_private_client, &scm_client_session_oid, 
-     "gzochi:client-session-oid");
-
-  initialize_binding 
-    (gzochi_private_data, &scm_make_managed_reference, 
-     "gzochi:make-managed-reference");
+  bind_scm ("gzochi private data", &scm_make_managed_reference, 
+	    "gzochi:make-managed-reference");
  
-  initialize_binding (gzochi_private_task, &scm_run_task, "gzochi:run-task");
+  bind_scm ("gzochi private task", &scm_run_task, "gzochi:run-task");
  
   gzochid_api_channel_init ();
   gzochid_api_data_init ();
@@ -928,4 +977,16 @@ void gzochid_scheme_channel_oid (SCM channel, mpz_t oid)
 void gzochid_scheme_initialize_bindings (void)
 {
   scm_with_guile (initialize_bindings, NULL);
+
+  gzochid_task_register_serialization 
+    ("scheme", &gzochid_scheme_task_serialization);
+}
+
+void gzochid_scheme_append_load_path (char *load_path)
+{
+  SCM scm_load_path = scm_module_variable 
+    (scm_c_resolve_module ("guile"), scm_from_locale_symbol ("%load-path"));
+  scm_variable_set_x 
+    (scm_load_path, scm_cons
+     (scm_from_locale_string (load_path), scm_variable_ref (scm_load_path)));
 }
