@@ -1,5 +1,5 @@
 ;; gzochi/private/app.scm: Private infrastructure for application support
-;; Copyright (C) 2011 Julian Graham
+;; Copyright (C) 2013 Julian Graham
 ;;
 ;; gzochi is free software: you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
@@ -42,22 +42,21 @@
 			       (gzochi:make-uniform-list-serialization 
 				gzochi:symbol-serialization)))
 	    (immutable data))
-    (nongenerative gzochi:callback)
     (protocol 
-     (lambda (n)
+     (lambda (p)
        (lambda (procedure module . args)
-	 (let ((p (n))) 
-	   (if (null? args)
-	       (p procedure module #f)
-	       (let ((arg (car args)))
-		 (or (and (gzochi:managed-record? arg) (null? (cdr args)))
-		     (raise
-		      (condition
-		       (make-assertion-violation)
-		       (make-message-condition
-			"Callback data must be a single managed record or #f."
-			))))
-		 (p procedure module arg)))))))
+	 (if (null? args)
+	     (p procedure module #f)
+	     (let ((arg (car args)))
+	       (or (and (or (gzochi:managed-record? arg) (not arg))
+			(null? (cdr args)))
+		   (raise
+		    (condition
+		     (make-assertion-violation)
+		     (make-message-condition
+		      "Callback data must be a single managed record or #f."
+		      ))))
+	       (p procedure module arg))))))
     (sealed #t))
 
   (define (gzochi:execute-initialized callback properties)
